@@ -1,10 +1,9 @@
 """
 functions.py - wraps functions of module _ldap
-written by Michael Stroeder <michael@stroeder.com>
 
-See http://python-ldap.sourceforge.net for details.
+See http://www.python-ldap.org/ for details.
 
-\$Id: functions.py,v 1.20 2007/03/22 23:12:36 stroeder Exp $
+\$Id: functions.py,v 1.30 2014/05/20 20:44:28 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -19,17 +18,18 @@ Basically calls into the LDAP lib are serialized by the module-wide
 lock _ldapmodule_lock.
 """
 
-__version__ = '0.1.2'
+from ldap import __version__
 
 __all__ = [
   'open','initialize','init',
   'explode_dn','explode_rdn',
   'get_option','set_option',
+  'escape_str',
 ]
 
-import sys,_ldap
+import sys,pprint,_ldap,ldap
 
-from ldap import _ldap_module_lock,LDAPError
+from ldap import LDAPError
 
 from ldap.dn import explode_dn,explode_rdn
 
@@ -38,34 +38,39 @@ from ldap.ldapobject import LDAPObject
 if __debug__:
   # Tracing is only supported in debugging mode
   import traceback
-  from ldap import _trace_level,_trace_file,_trace_stack_limit
 
-def _ldap_function_call(func,*args,**kwargs):
+
+def _ldap_function_call(lock,func,*args,**kwargs):
   """
-  Wrapper function which locks calls to func with via
-  module-wide ldap_lock
+  Wrapper function which locks and logs calls to function
+
+  lock
+      Instance of threading.Lock or compatible
+  func
+      Function to call with arguments passed in via *args and **kwargs
   """
+  if lock:
+    lock.acquire()
   if __debug__:
-    if _trace_level>=1:
-      _trace_file.write('*** %s.%s (%s,%s)\n' % (
-        '_ldap',repr(func),
-        repr(args),repr(kwargs)
+    if ldap._trace_level>=1:
+      ldap._trace_file.write('*** %s.%s %s\n' % (
+        '_ldap',func.__name__,
+        pprint.pformat((args,kwargs))
       ))
-      if _trace_level>=3:
-        traceback.print_stack(limit=_trace_stack_limit,file=_trace_file)
-  _ldap_module_lock.acquire()
+      if ldap._trace_level>=9:
+        traceback.print_stack(limit=ldap._trace_stack_limit,file=ldap._trace_file)
   try:
     try:
       result = func(*args,**kwargs)
     finally:
-      _ldap_module_lock.release()
+      if lock:
+        lock.release()
   except LDAPError,e:
-    if __debug__ and _trace_level>=2:
-      _trace_file.write('=> LDAPError: %s\n' % (str(e)))
+    if __debug__ and ldap._trace_level>=2:
+      ldap._trace_file.write('=> LDAPError: %s\n' % (str(e)))
     raise
-  if __debug__ and _trace_level>=2:
-    if result!=None and result!=(None,None):
-      _trace_file.write('=> result: %s\n' % (repr(result)))
+  if __debug__ and ldap._trace_level>=2:
+    ldap._trace_file.write('=> result:\n%s\n' % (pprint.pformat(result)))
   return result
 
 
@@ -73,7 +78,7 @@ def initialize(uri,trace_level=0,trace_file=sys.stdout,trace_stack_limit=None):
   """
   Return LDAPObject instance by opening LDAP connection to
   LDAP host specified by LDAP URL
-  
+
   Parameters:
   uri
         LDAP URL containing at least connection scheme and hostport,
@@ -91,7 +96,7 @@ def open(host,port=389,trace_level=0,trace_file=sys.stdout,trace_stack_limit=Non
   """
   Return LDAPObject instance by opening LDAP connection to
   specified LDAP host
-  
+
   Parameters:
   host
         LDAP host and port, e.g. localhost
@@ -103,6 +108,8 @@ def open(host,port=389,trace_level=0,trace_file=sys.stdout,trace_stack_limit=Non
         File object where to write the trace output to.
         Default is to use stdout.
   """
+  import warnings
+  warnings.warn('ldap.open() is deprecated! Use ldap.initialize() instead.', DeprecationWarning,2)
   return initialize('ldap://%s:%d' % (host,port),trace_level,trace_file,trace_stack_limit)
 
 init = open
@@ -114,13 +121,22 @@ def get_option(option):
 
   Get the value of an LDAP global option.
   """
-  return _ldap_function_call(_ldap.get_option,option)
+  return _ldap_function_call(None,_ldap.get_option,option)
 
 
 def set_option(option,invalue):
   """
   set_option(name, value)
-  
+
   Set the value of an LDAP global option.
   """
-  _ldap_function_call(_ldap.set_option,option,invalue)
+  return _ldap_function_call(None,_ldap.set_option,option,invalue)
+
+
+def escape_str(escape_func,s,*args):
+  """
+  Applies escape_func() to all items of `args' and returns a string based 
+  on format string `s'.
+  """
+  escape_args = map(escape_func,args)
+  return s % tuple(escape_args)
